###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Unit tests for Workflow Module UploadMC."""
# pylint: disable=protected-access, missing-docstring, invalid-name, line-too-long

import json
from pathlib import Path
from textwrap import dedent
import pytest

from DIRAC import S_OK
from LHCbDIRAC.Workflow.Modules.test.mock_Commons import (
    step_commons,
    wf_commons,
    prod_id,
    prod_job_id,
    wms_job_id,
    workflowStatus,
    step_id,
    step_number,
)
from LHCbDIRAC.Workflow.Modules.UploadMC import UploadMC


# Helpers
ERROR_FILE = f"{wms_job_id}_Errors_Gauss.json"
XML_SUMMARY = f"summaryGauss_{prod_id}_{prod_job_id}_1.xml"
GENERATOR_LOG = "GeneratorLog.xml"
PRMON = "prmon_Gauss.json"

GENERATED_GAUSS_SUMMARY = f"summaryGauss_{prod_id}_{prod_job_id}_1.json"
GENERATED_GENERATOR_LOG = f"GeneratorLog_{prod_id}_{prod_job_id}.json"


@pytest.fixture
def mc(mocker):
    """Setup AnalyseXMLSummaryNew module."""
    mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.RequestValidator")
    mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.ModuleBase._resolveInputVariables")

    # Setup module
    mc = UploadMC()

    yield mc

    Path(ERROR_FILE).unlink(missing_ok=True)
    Path(XML_SUMMARY).unlink(missing_ok=True)
    Path(GENERATOR_LOG).unlink(missing_ok=True)
    Path(PRMON).unlink(missing_ok=True)

    Path(GENERATED_GAUSS_SUMMARY).unlink(missing_ok=True)
    Path(GENERATED_GENERATOR_LOG).unlink(missing_ok=True)


# Tests
def test_MCUpload_success(mocker, mc):
    """Test MCUpload success."""
    mocker.patch("LHCbDIRAC.ProductionManagementSystem.Client.MCStatsClient.MCStatsClient.set", return_value=S_OK())
    mocker.patch("DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations.getValue", return_value=True)
    mcStatsClientSet = mocker.patch("LHCbDIRAC.ProductionManagementSystem.Client.MCStatsClient.MCStatsClient.set")

    # Create error file
    with open(ERROR_FILE, "w") as f:
        json.dump({"1": "error1", "2": "error2", "3": "error3"}, f)

    # Create XML summary
    summaryContent = dedent(
        """<?xml version="1.0" encoding="UTF-8"?>
        <summary version="1.0" xsi:noNamespaceSchemaLocation="$XMLSUMMARYBASEROOT/xml/XMLSummary.xsd"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
                <success>True</success>
                <step>finalize</step>
                <usage>
                        <stat unit="KB" useOf="MemoryMaximum">866104.0</stat>
                </usage>
                <input>
                        <file GUID="CCE96707-4BE9-E011-81CD-003048F35252" name="LFN:00012478_00000532_1.sim"
                        status="fail">200</file>
                        <file GUID="CCE96709-5BE9-E012-41BD-004048E36253" name="LFN:00012478_00000533_1.sim"
                        status="fail">200</file>
                </input>
                <output>
                        <file GUID="229BBEF1-66E9-E011-BBD0-003048F35252" name="PFN:00012478_00000532_2.xdigi"
                        status="full">200</file>
                </output>
                <counters>
                    <counter name="MCVeloHitPacker/# PackedData">281148</counter>
                    <counter name="MCPuVetoHitPacker/# PackedData">13019</counter>
                    <counter name="MCTTHitPacker/# PackedData">100850</counter>
                </counters>
        </summary>
        """
    )
    with open(XML_SUMMARY, "w") as f:
        f.write(summaryContent)

    # Create GeneratorLog
    generatorLogContent = dedent(
        """<?xml version="1.0"?>
        <generatorCounters>
        <version>1.1</version>
        <eventType>31111201</eventType>
        <counter name = "all events (including empty events)">
            <value> 17699 </value>
        </counter>
        <counter name = "generated events">
            <value> 17692 </value>
        </counter>
        <efficiency name = "generator level cut">
            <after> 39 </after>
            <before> 134 </before>
            <value> 0.29104 </value>
            <error> 0.039241 </error>
        </efficiency>
        <fraction name = "signal tau- in sample">
            <number> 41 </number>
            <value> 0.53247 </value>
            <error> 0.05686 </error>
        </fraction>
        <eventType>30000000</eventType>
        <method>GenerationNext.MinimumBias</method>
        <generator>Pythia8</generator>
        <method>GenerationPrev.MinimumBias</method>
        <generator>Pythia8</generator>
        <crosssection id = "101">
            <description> "non-diffractive" </description>
            <generated> 342 </generated>
            <value> 56.872 </value>
        </crosssection>
        </generatorCounters>
       """
    )

    with open(GENERATOR_LOG, "w") as f:
        f.write(generatorLogContent)

    # Create prmon file
    prmonContent = {}
    with open(PRMON, "w") as f:
        json.dump(prmonContent, f)

    # Call the execute method
    assert mc.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=step_commons[0],
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"]

    # Make sure the MCStatsClient is called as expected
    assert mcStatsClientSet.call_count == 4

    # Check the content of the generated gauss summary
    generatedSummaryGauss = Path(GENERATED_GAUSS_SUMMARY)
    assert generatedSummaryGauss.exists()
    with open(generatedSummaryGauss) as f:
        generatedSummaryGaussContent = json.load(f)

    expectedGeneratedSummaryGauss = {
        "Counters": {
            "MCVeloHitPacker": {"PackedData": 281148},
            "MCPuVetoHitPacker": {"PackedData": 13019},
            "MCTTHitPacker": {"PackedData": 100850},
            "ID": {"JobID": wms_job_id, "ProductionID": prod_id, "prod_job_id": prod_job_id},
        },
    }
    assert generatedSummaryGaussContent == expectedGeneratedSummaryGauss

    # Check the content of the generated generator log
    generatedGeneratorLog = Path(GENERATED_GENERATOR_LOG)
    assert generatedGeneratorLog.exists()
    with open(generatedGeneratorLog) as f:
        generatedGeneratorLogContent = json.load(f)

    expectedGeneratedGeneratorLog = {
        "generatorCounters": {
            "counter": {"all events (including empty events)": 17699, "generated events": 17692},
            "efficiency": {"generator level cut": {"after": 39, "before": 134, "error": 0.039241, "value": 0.29104}},
            "fraction": {"signal tau- in sample": {"number": 41, "error": 0.05686, "value": 0.53247}},
            "crossSection": {"non-diffractive": {"ID": 101, "generated": 342, "value": 56.872}},
            "method": {"GenerationNext.MinimumBias": "Pythia8", "GenerationPrev.MinimumBias": "Pythia8"},
            "ID": {"JobID": 12345, "ProductionID": "123", "prod_job_id": "00000456"},
        }
    }
    assert generatedGeneratorLogContent == expectedGeneratedGeneratorLog


def test_MCUpload_missingFiles(mocker, mc):
    """Test MCUpload success when there are missing files: it should upload the existing files."""
    mocker.patch("LHCbDIRAC.ProductionManagementSystem.Client.MCStatsClient.MCStatsClient.set", return_value=S_OK())
    mocker.patch("DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations.getValue", return_value=True)
    mcStatsClientSet = mocker.patch("LHCbDIRAC.ProductionManagementSystem.Client.MCStatsClient.MCStatsClient.set")

    # Create error file
    with open(ERROR_FILE, "w") as f:
        json.dump({"1": "error1", "2": "error2", "3": "error3"}, f)

    # Create prmon file
    prmonContent = {}
    with open(PRMON, "w") as f:
        json.dump(prmonContent, f)

    # Call the execute method
    assert mc.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=step_commons[0],
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"]

    assert mcStatsClientSet.call_count == 2
    assert not Path(GENERATED_GAUSS_SUMMARY).exists()
    assert not Path(GENERATED_GENERATOR_LOG).exists()


def test_MCUpload_disabledUpload(mocker, mc):
    """Test MCUpload success when uploads are disabled: it should not upload the files."""
    mocker.patch("LHCbDIRAC.ProductionManagementSystem.Client.MCStatsClient.MCStatsClient.set", return_value=S_OK())
    mcStatsClientSet = mocker.patch("LHCbDIRAC.ProductionManagementSystem.Client.MCStatsClient.MCStatsClient.set")
    # opsGetValue is mocked to return False: the upload is disabled
    mocker.patch("DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations.getValue", return_value=False)

    # Create error file
    with open(ERROR_FILE, "w") as f:
        json.dump({"1": "error1", "2": "error2", "3": "error3"}, f)

    # Create prmon file
    prmonContent = {}
    with open(PRMON, "w") as f:
        json.dump(prmonContent, f)

    # Call the execute method
    assert mc.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=step_commons[0],
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"]

    assert not mcStatsClientSet.called
    assert not Path(GENERATED_GAUSS_SUMMARY).exists()
    assert not Path(GENERATED_GENERATOR_LOG).exists()


def test_MCUpload_malformedXMLSummary(mocker, mc):
    """Test MCUpload when there is a malformed input: it should upload the other files even if there is an issue."""
    mocker.patch("LHCbDIRAC.ProductionManagementSystem.Client.MCStatsClient.MCStatsClient.set", return_value=S_OK())
    mcStatsClientSet = mocker.patch("LHCbDIRAC.ProductionManagementSystem.Client.MCStatsClient.MCStatsClient.set")
    mocker.patch("DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations.getValue", return_value=True)

    # Create error file
    with open(ERROR_FILE, "w") as f:
        json.dump({"1": "error1", "2": "error2", "3": "error3"}, f)

    # Create a malformed XML summary: there is no counter
    summaryContent = dedent(
        """<?xml version="1.0" encoding="UTF-8"?>
        <summary version="1.0" xsi:noNamespaceSchemaLocation="$XMLSUMMARYBASEROOT/xml/XMLSummary.xsd"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
                <success>True</success>
                <step>finalize</step>
                <usage>
                        <stat unit="KB" useOf="MemoryMaximum">866104.0</stat>
                </usage>
                <input>
                        <file GUID="CCE96707-4BE9-E011-81CD-003048F35252" name="LFN:00012478_00000532_1.sim"
                        status="fail">200</file>
                        <file GUID="CCE96709-5BE9-E012-41BD-004048E36253" name="LFN:00012478_00000533_1.sim"
                        status="fail">200</file>
                </input>
                <output>
                        <file GUID="229BBEF1-66E9-E011-BBD0-003048F35252" name="PFN:00012478_00000532_2.xdigi"
                        status="full">200</file>
                </output>
        </summary>
        """
    )
    with open(XML_SUMMARY, "w") as f:
        f.write(summaryContent)

    # Create prmon file
    prmonContent = {}
    with open(PRMON, "w") as f:
        json.dump(prmonContent, f)

    # Call the execute method
    assert mc.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=step_commons[0],
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"]

    # Make sure the MCStatsClient is called 2 times and not 3
    assert mcStatsClientSet.call_count == 2
    assert not Path(GENERATED_GAUSS_SUMMARY).exists()
    assert not Path(GENERATED_GENERATOR_LOG).exists()


def test_MCUpload_noInputData(mocker, mc):
    """Test MCUpload success when there is no relevant input data."""
    mocker.patch("LHCbDIRAC.ProductionManagementSystem.Client.MCStatsClient.MCStatsClient.set", return_value=S_OK())
    mocker.patch("DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations.getValue", return_value=True)
    mcStatsClientSet = mocker.patch("LHCbDIRAC.ProductionManagementSystem.Client.MCStatsClient.MCStatsClient.set")

    assert mc.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=step_commons[0],
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"]

    assert not mcStatsClientSet.called
    assert not Path(GENERATED_GAUSS_SUMMARY).exists()
    assert not Path(GENERATED_GENERATOR_LOG).exists()



###############################################################################
# (c) Copyright 2024 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Unit tests for Workflow Module StepAccounting."""

from textwrap import dedent
import pytest
from pathlib import Path

from DIRAC import S_OK, S_ERROR
from LHCbDIRAC.Core.Utilities.XMLSummaries import XMLSummary
from LHCbDIRAC.Workflow.Modules.StepAccounting import StepAccounting
from LHCbDIRAC.Workflow.Modules.test.mock_Commons import (
    prod_id,
    prod_job_id,
    wms_job_id,
    workflowStatus,
    step_id,
    step_number,
    wf_commons,
)


# Helper Functions
XML_SUMMARY_FILE = "XMLSummary.xml"


def prepare_XMLSummary_file(xml_summary, content):
    with open(xml_summary, "w") as f:
        f.write(content)
    return XMLSummary(xml_summary)


@pytest.fixture
def accounting(mocker):
    """Fixture for StepAccounting module."""
    mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.RequestValidator")
    mocker.patch("LHCbDIRAC.Workflow.Modules.StepAccounting.StepAccounting._resolveInputVariables")

    step_acc = StepAccounting()
    step_acc.jobID = wms_job_id
    mocker.patch.object(step_acc, "dsc", autospec=True)

    yield step_acc

    Path(XML_SUMMARY_FILE).unlink(missing_ok=True)


# Test Scenarios
def test_accounting_success(accounting):
    """Test successful execution of StepAccounting module."""
    # Mock the StepAccounting module
    accounting.step_commons = {
        "applicationName": "Gauss",
    }
    xml_content = dedent(
        """<?xml version="1.0" encoding="UTF-8"?>
        <summary version="1.0" xsi:noNamespaceSchemaLocation="$XMLSUMMARYBASEROOT/xml/XMLSummary.xsd"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
                <success>True</success>
                <step>finalize</step>
                <usage>
                        <stat unit="KB" useOf="MemoryMaximum">866104.0</stat>
                </usage>
                <input>
                        <file GUID="CCE96707-4BE9-E011-81CD-003048F35252" name="LFN:00012478_00000532_1.sim"
                        status="full">200</file>
                </input>
                <output>
                        <file GUID="229BBEF1-66E9-E011-BBD0-003048F35252" name="PFN:00012478_00000532_2.xdigi"
                        status="full">200</file>
                </output>
        </summary>
        """
    )
    accounting.step_commons["XMLSummary_o"] = prepare_XMLSummary_file(XML_SUMMARY_FILE, xml_content)

    accounting.BKstepID = "12345"
    accounting.stepProcPass = "Sim09m"
    accounting.eventType = "23103003"
    accounting.stepStat = "Done"

    assert accounting.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=accounting.step_commons,
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"], "Execution should succeed."

    # Make sure the dsc was called
    accounting.dsc.addRegister.assert_called_once()


def test_accounting_noApplicationName_success(accounting):
    """Test StepAccounting when there is no application name in step commons."""
    # Mock the StepAccounting module
    accounting.step_commons = {}

    xml_content = dedent(
        """<?xml version="1.0" encoding="UTF-8"?>
        <summary version="1.0" xsi:noNamespaceSchemaLocation="$XMLSUMMARYBASEROOT/xml/XMLSummary.xsd"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
                <success>True</success>
                <step>finalize</step>
                <usage>
                        <stat unit="KB" useOf="MemoryMaximum">866104.0</stat>
                </usage>
                <input>
                        <file GUID="CCE96707-4BE9-E011-81CD-003048F35252" name="LFN:00012478_00000532_1.sim"
                        status="full">200</file>
                </input>
                <output>
                        <file GUID="229BBEF1-66E9-E011-BBD0-003048F35252" name="PFN:00012478_00000532_2.xdigi"
                        status="full">200</file>
                </output>
        </summary>
        """
    )
    accounting.step_commons["XMLSummary_o"] = prepare_XMLSummary_file(XML_SUMMARY_FILE, xml_content)

    assert accounting.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=accounting.step_commons,
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"]

    assert not accounting.dsc.addRegister.called, "No accounting data should be added."


def test_accounting_incompleteData(accounting):
    """Test successful execution of StepAccounting module."""
    # Mock the StepAccounting module
    accounting.step_commons = {
        "applicationName": "Gauss",
    }
    xml_content = dedent(
        """<?xml version="1.0" encoding="UTF-8"?>
        <summary version="1.0" xsi:noNamespaceSchemaLocation="$XMLSUMMARYBASEROOT/xml/XMLSummary.xsd"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
                <success>True</success>
                <step>finalize</step>
                <usage>
                        <stat unit="KB" useOf="MemoryMaximum">866104.0</stat>
                </usage>
                <input>
                        <file GUID="CCE96707-4BE9-E011-81CD-003048F35252" name="LFN:00012478_00000532_1.sim"
                        status="full">200</file>
                </input>
                <output>
                        <file GUID="229BBEF1-66E9-E011-BBD0-003048F35252" name="PFN:00012478_00000532_2.xdigi"
                        status="full">200</file>
                </output>
        </summary>
        """
    )
    accounting.step_commons["XMLSummary_o"] = prepare_XMLSummary_file(XML_SUMMARY_FILE, xml_content)

    assert not accounting.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=accounting.step_commons,
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"], "Execution should not succeed."

    assert not accounting.dsc.addRegister.called, "No accounting data should be added."


def test_accounting_previousError_fail(accounting):
    """Test StepAccounting with an intentional failure."""
    # Mock the StepAccounting module
    accounting.step_commons = {
        "applicationName": "Gauss",
    }
    xml_content = dedent(
        """<?xml version="1.0" encoding="UTF-8"?>
        <summary version="1.0" xsi:noNamespaceSchemaLocation="$XMLSUMMARYBASEROOT/xml/XMLSummary.xsd"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
                <success>True</success>
                <step>finalize</step>
                <usage>
                        <stat unit="KB" useOf="MemoryMaximum">866104.0</stat>
                </usage>
                <input>
                        <file GUID="CCE96707-4BE9-E011-81CD-003048F35252" name="LFN:00012478_00000532_1.sim"
                        status="full">200</file>
                </input>
                <output>
                        <file GUID="229BBEF1-66E9-E011-BBD0-003048F35252" name="PFN:00012478_00000532_2.xdigi"
                        status="full">200</file>
                </output>
        </summary>
        """
    )
    accounting.step_commons["XMLSummary_o"] = prepare_XMLSummary_file(XML_SUMMARY_FILE, xml_content)

    accounting.BKstepID = "12345"
    accounting.stepProcPass = "Sim09m"
    accounting.eventType = "23103003"

    # Intentional error
    accounting.stepStat = "Failed"

    assert accounting.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=accounting.step_commons,
        step_number=step_number,
        step_id=step_id,
        # Intentional error
        stepStatus=S_ERROR(),
    )["OK"]

    assert accounting.dsc.addRegister.called, "Accounting data should be added."



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Test for CreateDataFile module."""
from pathlib import Path
from textwrap import dedent
import pytest
from LHCbDIRAC.Workflow.Modules.CreateDataFile import CreateDataFile
from DIRAC import S_OK, S_ERROR
from LHCbDIRAC.Workflow.Modules.test.mock_Commons import (
    prod_id,
    prod_job_id,
    wms_job_id,
    workflowStatus,
    step_id,
    step_number,
    wf_commons,
)


@pytest.fixture
def cdf(mocker):
    """Fixture for creating an instance of CreateDataFile module."""
    mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.RequestValidator")
    mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.ModuleBase._resolveInputVariables")
    mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.ModuleBase._resolveInputStep")

    cdf = CreateDataFile()

    yield cdf

    Path(cdf.gangaFileName).unlink(missing_ok=True)


def test_createDataFile_success(cdf):
    """Test CreateDataFile module successfully creates a data file."""
    cdf.inputDataList = [
        "/lhcb/MC/Dev/XDIGI/00211521/0000/00211521_00000489_1.dst",
        "/lhcb/MC/Dev/XDIGI/00211521/0000/00211521_00000489_2.dst",
    ]

    assert cdf.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=cdf.step_commons,
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"], "Execution should succeed."

    assert Path(cdf.gangaFileName).exists(), "Ganga data file should exist."
    with open(cdf.gangaFileName) as f:
        content = f.read()

    expectedContent = dedent(
        """from Gaudi.Configuration import * \nfrom GaudiConf import IOHelper
IOHelper().inputFiles([
'LFN:/lhcb/MC/Dev/XDIGI/00211521/0000/00211521_00000489_1.dst',
'LFN:/lhcb/MC/Dev/XDIGI/00211521/0000/00211521_00000489_2.dst',
], clear=True)

FileCatalog().Catalogs += [ 'xmlcatalog_file:pool_xml_catalog.xml' ]
"""
    )

    assert expectedContent in content


def test_createDataFile_existingData(cdf):
    """Test CreateDataFile module successfully creates a data file."""
    # data.py already exists
    with open(cdf.gangaFileName, "w") as f:
        f.write("Some existing content.")

    cdf.inputDataList = [
        "/lhcb/MC/Dev/XDIGI/00211521/0000/00211521_00000489_1.dst",
        "/lhcb/MC/Dev/XDIGI/00211521/0000/00211521_00000489_2.dst",
    ]

    assert cdf.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=cdf.step_commons,
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"], "Execution should succeed."

    assert Path(cdf.gangaFileName).exists(), "Ganga data file should be created."
    with open(cdf.gangaFileName) as f:
        content = f.read()

    expectedContent = dedent(
        """from Gaudi.Configuration import * \nfrom GaudiConf import IOHelper
IOHelper().inputFiles([
'LFN:/lhcb/MC/Dev/XDIGI/00211521/0000/00211521_00000489_1.dst',
'LFN:/lhcb/MC/Dev/XDIGI/00211521/0000/00211521_00000489_2.dst',
], clear=True)

FileCatalog().Catalogs += [ 'xmlcatalog_file:pool_xml_catalog.xml' ]
"""
    )

    assert expectedContent in content, "Ganga data file should contain the expected content."


def test_createDataFile_noInputData(cdf):
    """Test CreateDataFile when there is no input data: it should exit without data.py."""
    # No input data
    cdf.inputDataList = []

    assert cdf.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=cdf.step_commons,
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"], "Execution should succeed."

    assert not Path(cdf.gangaFileName).exists(), "Ganga data file should not be created."


def test_createDataFile_previousError(cdf):
    """Test CreateDataFile module when there is a previous error: it should not create data.py."""
    cdf.inputDataList = [
        "/lhcb/MC/Dev/XDIGI/00211521/0000/00211521_00000489_1.dst",
        "/lhcb/MC/Dev/XDIGI/00211521/0000/00211521_00000489_2.dst",
    ]

    assert cdf.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=cdf.step_commons,
        step_number=step_number,
        step_id=step_id,
        # Intentionally setting stepStatus to S_ERROR
        stepStatus=S_ERROR(),
    )["OK"], "Execution should succeed."

    assert not Path(cdf.gangaFileName).exists(), "Ganga data file should not be created."



###############################################################################
# (c) Copyright 2024 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Unit tests for Workflow Module FailoverRequest."""

import json
from pathlib import Path
import pytest

from DIRAC import S_OK, S_ERROR
from DIRAC.RequestManagementSystem.Client.Request import Request
from DIRAC.TransformationSystem.Client.FileReport import FileReport
from DIRAC.WorkloadManagementSystem.Client.JobReport import JobReport

from LHCbDIRAC.Workflow.Modules.FailoverRequest import FailoverRequest
from LHCbDIRAC.Workflow.Modules.test.mock_Commons import (
    prod_id,
    prod_job_id,
    wms_job_id,
    workflowStatus,
    step_id,
    step_number,
    wf_commons,
)


# Helper Functions
REQUEST_JSON = f"{prod_id}_{prod_job_id}_request.json"


@pytest.fixture
def fr(mocker):
    """Fixture for FailoverRequest module."""
    mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.RequestValidator")
    mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.ModuleBase._resolveInputVariables")
    mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.ModuleBase._resolveInputStep")

    fr = FailoverRequest()
    fr.fileReport = FileReport()
    fr.jobReport = JobReport(wms_job_id)
    fr.request = Request()

    yield fr

    Path(REQUEST_JSON).unlink(missing_ok=True)


# Test Scenarios
def test_failoverRequest_success(mocker, fr):
    """Test successful execution of FailoverRequest module."""
    problematicFiles = [
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000287_1.ew.dst",
    ]
    mocker.patch("DIRAC.TransformationSystem.Client.FileReport.FileReport.getFiles", side_effect=[problematicFiles, []])
    mocker.patch("DIRAC.TransformationSystem.Client.FileReport.FileReport.commit", return_value=S_OK("Anything"))
    mocker.patch.object(fr.fileReport, "setFileStatus")
    mocker.patch.object(fr.jobReport, "setApplicationStatus")

    fr.inputDataList = [
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
        "/lhcb/data/2011/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
    ] + problematicFiles

    # Execute the module
    assert fr.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=fr.step_commons,
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"], "Execution should succeed."

    # Check the FileReport calls: the problematic file should not appear
    # The input files should be set to "Processed"
    assert fr.fileReport.setFileStatus.call_count == 2
    args = fr.fileReport.setFileStatus.call_args_list
    assert args[0][0][0] == int(prod_id)
    assert args[0][0][1] == fr.inputDataList[0]
    assert args[0][0][2] == "Processed"

    assert args[1][0][0] == int(prod_id)
    assert args[1][0][1] == fr.inputDataList[1]
    assert args[1][0][2] == "Processed"

    # Make sure the appliction is successfully finished
    assert fr.jobReport.setApplicationStatus.call_count == 1
    assert fr.jobReport.setApplicationStatus.call_args[0][0] == "Job Finished Successfully"

    # Make sure the forward DISET is not generated
    requestDict = json.loads(fr.request.toJSON()["Value"])
    operations = requestDict["Operations"]
    assert len(operations) == 0

    # Make sure the request json does not exists
    assert not Path(REQUEST_JSON).exists()


def test_failoverRequest_commitFailure1(mocker, fr):
    """Test execution of FailoverRequest module when the fileReport.commit() fails:
    In this context, the second call to commit() will work, so the request should not be generated.
    """
    problematicFiles = [
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000287_1.ew.dst",
    ]
    # Both calla to getFiles() will return the problematic files because the commit did not work
    mocker.patch(
        "DIRAC.TransformationSystem.Client.FileReport.FileReport.getFiles",
        side_effect=[problematicFiles, problematicFiles],
    )
    mocker.patch(
        "DIRAC.TransformationSystem.Client.FileReport.FileReport.commit", side_effect=[S_ERROR("Error"), S_OK(None)]
    )
    mocker.patch.object(fr.fileReport, "setFileStatus")
    mocker.patch.object(fr.jobReport, "setApplicationStatus")

    fr.inputDataList = [
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
        "/lhcb/data/2011/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
    ] + problematicFiles

    # Execute the module
    assert fr.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=fr.step_commons,
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"], "Execution should succeed."

    # Check the FileReport calls: the problematic file should not appear
    # The input files should be set to "Processed"
    assert fr.fileReport.setFileStatus.call_count == 2
    args = fr.fileReport.setFileStatus.call_args_list
    assert args[0][0][0] == int(prod_id)
    assert args[0][0][1] == fr.inputDataList[0]
    assert args[0][0][2] == "Processed"

    assert args[1][0][0] == int(prod_id)
    assert args[1][0][1] == fr.inputDataList[1]
    assert args[1][0][2] == "Processed"

    # Make sure the appliction is successfully finished
    assert fr.jobReport.setApplicationStatus.call_count == 1
    assert fr.jobReport.setApplicationStatus.call_args[0][0] == "Job Finished Successfully"

    # Make sure the forward DISET is generated
    requestDict = json.loads(fr.request.toJSON()["Value"])
    operations = requestDict["Operations"]
    assert len(operations) == 0

    # Make sure the request json does not exists
    assert not Path(REQUEST_JSON).exists()


def test_failoverRequest_commitFailure2(mocker, fr):
    """Test execution of FailoverRequest module when the fileReport.commit() fails:
    In this context, the second call to commit() will fail, so the request should be generated.
    """
    problematicFiles = [
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000287_1.ew.dst",
    ]
    # Both calla to getFiles() will return the problematic files because the commit did not work
    mocker.patch(
        "DIRAC.TransformationSystem.Client.FileReport.FileReport.getFiles",
        side_effect=[problematicFiles, problematicFiles],
    )
    mocker.patch(
        "DIRAC.TransformationSystem.Client.FileReport.FileReport.commit",
        side_effect=[S_ERROR("Error"), S_ERROR("Error")],
    )
    mocker.patch.object(fr.fileReport, "setFileStatus")
    mocker.patch.object(fr.jobReport, "setApplicationStatus")

    fr.inputDataList = [
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
        "/lhcb/data/2011/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
    ] + problematicFiles

    # Execute the module
    assert fr.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=fr.step_commons,
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"], "Execution should succeed."

    # Check the FileReport calls: the problematic file should not appear
    # The input files should be set to "Processed"
    assert fr.fileReport.setFileStatus.call_count == 2
    args = fr.fileReport.setFileStatus.call_args_list
    assert args[0][0][0] == int(prod_id)
    assert args[0][0][1] == fr.inputDataList[0]
    assert args[0][0][2] == "Processed"

    assert args[1][0][0] == int(prod_id)
    assert args[1][0][1] == fr.inputDataList[1]
    assert args[1][0][2] == "Processed"

    # Make sure the appliction is successfully finished
    assert fr.jobReport.setApplicationStatus.call_count == 1
    assert fr.jobReport.setApplicationStatus.call_args[0][0] == "Job Finished Successfully"

    # Make sure the forward DISET is generated
    requestDict = json.loads(fr.request.toJSON()["Value"])
    operations = requestDict["Operations"]

    assert len(operations) == 1
    assert operations[0]["Type"] == "SetFileStatus"

    # Make sure the request json does not exists
    assert Path(REQUEST_JSON).exists()


def test_failoverRequest_previousError_fail(mocker, fr):
    """Test FailoverRequest with an intentional failure."""
    problematicFiles = [
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000287_1.ew.dst",
    ]
    mocker.patch("DIRAC.TransformationSystem.Client.FileReport.FileReport.getFiles", side_effect=[problematicFiles, []])
    mocker.patch("DIRAC.TransformationSystem.Client.FileReport.FileReport.commit", return_value=S_OK("Anything"))
    mocker.patch.object(fr.fileReport, "setFileStatus")
    mocker.patch.object(fr.jobReport, "setApplicationStatus")

    fr.inputDataList = [
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
        "/lhcb/data/2011/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
    ] + problematicFiles

    # Execute the module
    assert fr.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=fr.step_commons,
        step_number=step_number,
        step_id=step_id,
        # Intentional error
        stepStatus=S_ERROR(),
    )["OK"], "Execution should succeed."

    # Check the FileReport calls: the problematic file should not appear
    # The input files should be set to "Unused"
    assert fr.fileReport.setFileStatus.call_count == 2
    args = fr.fileReport.setFileStatus.call_args_list
    assert args[0][0][0] == int(prod_id)
    assert args[0][0][1] == fr.inputDataList[0]
    assert args[0][0][2] == "Unused"

    assert args[1][0][0] == int(prod_id)
    assert args[1][0][1] == fr.inputDataList[1]
    assert args[1][0][2] == "Unused"

    # Make sure the appliction is not reported as a success
    assert fr.jobReport.setApplicationStatus.call_count == 0

    # Make sure the forward DISET is not generated
    requestDict = json.loads(fr.request.toJSON()["Value"])
    operations = requestDict["Operations"]
    assert len(operations) == 0

    # Make sure the request json does not exists
    assert not Path(REQUEST_JSON).exists()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""The LHCb AncestorFiles executor queries the Bookkeeping catalogue for ancestor
files if the JDL parameter AncestorDepth is specified.

The ancestor files are subsequently added to the existing input data
requirement of the job.
"""

from DIRAC import S_OK, S_ERROR
from DIRAC.WorkloadManagementSystem.Executor.Base.OptimizerExecutor import OptimizerExecutor
from DIRAC.WorkloadManagementSystem.DB.JobDB import JobDB
from LHCbDIRAC.BookkeepingSystem.Client.BookkeepingClient import BookkeepingClient


class AncestorFiles(OptimizerExecutor):
    """
    The specific Optimizer must provide the following methods:
      - initializeOptimizer() before each execution cycle
      - optimizeJob() - the main method called for each job
    """

    @classmethod
    def initializeOptimizer(cls):
        """Initialize specific parameters for AncestorFiles executor."""
        cls.bk = BookkeepingClient()
        cls.jobDB = JobDB()
        return S_OK()

    def optimizeJob(self, jid, jobState):
        """This method checks the input data with ancestors.

        The original job JDL is always extracted to obtain the input data,
        therefore rescheduled jobs will not recursively search for ancestors
        of ancestors etc.
        """
        inputData = []
        if jobState.lookupAttribute("InputData"):
            inputData = jobState.getListFromExpression("InputData")

        if not jobState.lookupAttribute("AncestorDepth"):
            self.log.warn(f"No AncestorDepth requirement found for job {jid}")
            return S_ERROR("AncestorDepth Not Found")

        ancestorDepth = jobState.getAttributeInt("AncestorDepth")

        if ancestorDepth == 0:
            return S_OK("Null AncestorDepth specified")

        self.log.info(
            f"Job {jid} has {len(inputData)} input data files and specified ancestor depth of {ancestorDepth}"
        )
        result = self.__getInputDataWithAncestors(jid, inputData, ancestorDepth)
        if not result["OK"]:
            return result

        newInputData = result["Value"]

        jobState.insertAttributeVectorString("InputData", newInputData)
        newJDL = jobState.asJDL()
        if not (result := self.__setJobInputData(jid, newJDL, newInputData))["OK"]:
            return result

        return self.setNextOptimizer()

    ############################################################################
    def __getInputDataWithAncestors(self, job, inputData, ancestorDepth):
        """Extend the list of LFNs with the LFNs for their ancestor files for the
        generation depth specified in the job JDL."""
        inputData = [i.replace("LFN:", "") for i in inputData]
        result = self.bk.getFileAncestors(inputData, ancestorDepth, replica=True)
        if not result["OK"]:
            self.log.warn(result["Message"])
            return result

        ancestors = [anc["FileName"] for ancList in result["Value"]["Successful"].values() for anc in ancList]

        return S_OK(ancestors + inputData)

    def __setJobInputData(self, job, jdl, inputData):
        """Sets the new job input data requirement including ancestor files."""
        inputData = [i.replace("LFN:", "") for i in inputData]

        if not (result := self.jobDB.setInputData(job, inputData))["OK"]:
            self.log.error(result["Message"])
            return result

        if not (result := self.jobDB.setJobJDL(job, jdl))["OK"]:
            self.log.error(result["Message"])
            return result

        return S_OK("Job updated")



###############################################################################
# (c) Copyright 2024 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Unit tests for Workflow Module RemoveInputData."""

import json
import pytest

from DIRAC import S_OK, S_ERROR
from DIRAC.RequestManagementSystem.Client.Request import Request
from LHCbDIRAC.Workflow.Modules.RemoveInputData import RemoveInputData
from LHCbDIRAC.Workflow.Modules.test.mock_Commons import (
    prod_id,
    prod_job_id,
    wms_job_id,
    workflowStatus,
    step_id,
    step_number,
    wf_commons,
)


# Helper Functions
@pytest.fixture
def rid(mocker):
    """Fixture for StepAccounting module."""
    mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.RequestValidator")
    mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.ModuleBase._resolveInputVariables")

    rid = RemoveInputData()
    rid.request = Request()

    yield rid


# Test Scenarios
def test_removeInputData_success(mocker, rid):
    """Test successful execution of RemoveInputData module."""
    rid.inputDataList = [
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000287_1.ew.dst",
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
        "/lhcb/data/2011/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
    ]
    successful = dict.fromkeys((lfn for lfn in rid.inputDataList), True)

    mocker.patch(
        "DIRAC.DataManagementSystem.Client.DataManager.DataManager.removeFile",
        return_value=S_OK({"Successful": successful, "Failed": {}}),
    )

    assert rid.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=rid.step_commons,
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"], "Execution should succeed."

    rid.dataManager.removeFile.assert_called_once()
    args, _ = rid.dataManager.removeFile.call_args
    assert args[0] == rid.inputDataList

    assert rid.request.isEmpty()


def test_removeInputData_failedRemoval(mocker, rid):
    """Test RemoveInputData when the removal of input data fails."""
    successInputDataList = [
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000287_1.ew.dst",
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
        "/lhcb/data/2011/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
    ]
    failedInputDataList = [
        "/lhcb/data/2011/EW.DST/00008380/0000/00008380_00000285_1.ew.dst",
        "/lhcb/data/2011/EW.DST/00008380/0000/00008380_00000286_1.ew.dst",
    ]
    rid.inputDataList = successInputDataList + failedInputDataList
    successful = dict.fromkeys((lfn for lfn in successInputDataList), True)
    failed = dict.fromkeys((lfn for lfn in failedInputDataList), "Failed to remove file")

    mocker.patch(
        "DIRAC.DataManagementSystem.Client.DataManager.DataManager.removeFile",
        return_value=S_OK({"Successful": successful, "Failed": failed}),
    )

    assert rid.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=rid.step_commons,
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"], "Execution should succeed."

    rid.dataManager.removeFile.assert_called_once()
    args, _ = rid.dataManager.removeFile.call_args
    assert args[0] == rid.inputDataList

    assert not rid.request.isEmpty()

    requestDict = json.loads(rid.request.toJSON()["Value"])
    operations = requestDict["Operations"]
    for operation in operations:
        assert operation["Type"] == "RemoveFile"
        assert len(operation["Files"]) == 1
        assert operation["Files"][0]["LFN"] in failedInputDataList


def test_removeInputData_errorRemoval(mocker, rid):
    """Test Test RemoveInputData when the removal of input data returns an error."""
    rid.inputDataList = [
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000287_1.ew.dst",
        "/lhcb/data/2010/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
        "/lhcb/data/2011/EW.DST/00008380/0000/00008380_00000281_1.ew.dst",
    ]

    mocker.patch("DIRAC.DataManagementSystem.Client.DataManager.DataManager.removeFile", return_value=S_ERROR("Error"))
    mocker.patch("LHCbDIRAC.Workflow.Modules.ModuleBase.ModuleBase.setApplicationStatus")

    assert rid.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=rid.step_commons,
        step_number=step_number,
        step_id=step_id,
        stepStatus=S_OK(),
    )["OK"], "Execution should succeed."

    rid.dataManager.removeFile.assert_called_once()
    args, _ = rid.dataManager.removeFile.call_args
    assert args[0] == rid.inputDataList

    assert not rid.request.isEmpty()

    requestDict = json.loads(rid.request.toJSON()["Value"])
    operations = requestDict["Operations"]
    for operation in operations:
        assert operation["Type"] == "RemoveFile"
        assert len(operation["Files"]) == 1
        assert operation["Files"][0]["LFN"] in rid.inputDataList


def test_removeInputData_previousError_fail(mocker, rid):
    """Test RemoveInputData with an intentional failure from a previous step."""
    mocker.patch("DIRAC.DataManagementSystem.Client.DataManager.DataManager.removeFile")
    assert rid.execute(
        production_id=prod_id,
        prod_job_id=prod_job_id,
        wms_job_id=wms_job_id,
        workflowStatus=workflowStatus,
        wf_commons=wf_commons[0],
        step_commons=rid.step_commons,
        step_number=step_number,
        step_id=step_id,
        # Intentional error
        stepStatus=S_ERROR(),
    )["OK"]

    rid.dataManager.removeFile.assert_not_called()
    assert rid.request.isEmpty()
